# LongDefi Smart Account Sdk

## Installation

```bash
npm install @longdefi/smart-account-sdk viem@^1.51.1
```

## Usage

### `SmartAccountV1Provider` instance:

```typescript
import { SmartAccountV1Provider } from "@longdefi/smart-account-sdk";

const smartAccountV1Provider = new SmartAccountV1Provider(publicClient);
```

### Swap request with **new** wallet

```typescript
const initSmartAccountInput = {
  owner: ownerSmartAccount.account.address,
  salt: BigInt(0),
};
const singlePathSwapInput = {
  tokenIn: token1.address,
  tokenOut: token0.address,
  fee: 3000,
  deadline: BigInt(2 ** 255),
  amountIn: BigInt(10 ** 18),
  amountOutMinimum: BigInt(0),
  sqrtPriceLimitX96: BigInt(0),
};

const { smartAccount, userOpHash, request } = await smartAccountV1Provider.createSwapRequest({
  dex: "uniswapV3",
  swapInput: singlePathSwapInput,
  gasless: true,
  initSmartAccountInput,
});

// Using `wagmi` to sign request
const signature = await signMessage(config, {
  message: { raw: userOpHash },
});

const userOp1 = { ...request, signature };

// when conditions is met, send userOp to `entrypoint`
const tx = await bundler.writeContract({
  abi: entrypointERC4337.abi,
  address: entrypointERC4337.address,
  functionName: "handleOps",
  args: [[userOp1, userOp2, userOp3], bundler.account.address],
});
```

### Swap request with **existed** wallet

```typescript
const singlePathSwapInput = {
  tokenIn: token1.address,
  tokenOut: token0.address,
  fee: 3000,
  deadline: BigInt(2 ** 255),
  amountIn: BigInt(20),
  amountOutMinimum: BigInt(0),
  sqrtPriceLimitX96: BigInt(0),
};
const gasless = true;
const { userOpHash, request } = await smartAccountV1Provider.createSwapRequest({
  smartAccount: smartAccountV1.address,
  dex: "uniswapV3",
  swapInput: singlePathSwapInput,
  gasless,
});

// Using `wagmi` to sign request
const signature = await signMessage(config, {
  message: { raw: userOpHash },
});

const userOp = { ...request, signature };
```

### Price Utils

```typescript
const publicClient = createPublicClient({ chain: mainnet, transport: http() });

// The order of tokens is not important
const price = await convertSqrtX96toPrice(publicClient, [token0, token1], sqrtPriceX96);

const sqrtX96 = await convertPriceToSqrtX96(publicClient, [token0, token1], price);
```

## Supported Chains

- Mumbai
